"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = __importStar(require("http"));
const net = __importStar(require("net"));
const events_1 = require("events");
const httpServer_1 = __importDefault(require("./httpServer"));
const httpsServer_1 = __importDefault(require("./httpsServer"));
class BridgeServer extends events_1.EventEmitter {
    constructor(interceptHandler, certAuthority) {
        super();
        this.bridgeServer = this.createBridgeServer();
        this.httpServer = new httpServer_1.default(interceptHandler);
        if (certAuthority) {
            const httpsServer = new httpsServer_1.default(certAuthority, interceptHandler);
            this.httpsServer = httpsServer;
            this.createHttpsBridge(httpsServer);
        }
        this.forwardErrors();
    }
    createBridgeServer() {
        const bridgeServer = http.createServer((req, res) => {
            const httpServerAddress = this.httpServer.address;
            const requestTarget = {
                host: 'localhost',
                port: httpServerAddress.port,
                path: req.url,
                method: req.method,
                headers: req.headers
            };
            const forwardRequest = http.request(requestTarget, (httpServerResponse) => {
                res.writeHead(httpServerResponse.statusCode, httpServerResponse.headers);
                httpServerResponse.pipe(res);
            });
            req.pipe(forwardRequest);
            forwardRequest.on('error', (err) => this.emit('error', err));
        });
        return bridgeServer;
    }
    forwardErrors() {
        var _a;
        this.bridgeServer.on('error', (error) => this.emit('error', error));
        this.httpServer.on('error', (error) => this.emit('error', error));
        (_a = this.httpsServer) === null || _a === void 0 ? void 0 : _a.on('error', (error) => this.emit('error', error));
    }
    createHttpsBridge(httpsServer) {
        this.bridgeServer.on('connect', (request, clientSocket, _) => {
            const address = httpsServer.address;
            // Creates TCP connection to the target server
            let serverSocket = net.connect(address.port, address.address, () => {
                const successConnection = Buffer.from(`HTTP/${request.httpVersion} 200 Connection Established\r\n\r\n`, 'utf-8');
                // Tell the client (the HTTP server) that the connection was successfully established
                clientSocket.write(successConnection);
                clientSocket
                    .pipe(serverSocket)
                    .pipe(clientSocket);
            });
            serverSocket.on('error', (err) => this.emit('error', err));
            clientSocket.on('error', () => { }); // Ignore ECONNRESET ERRORS THAT ARE HANDLED BY THE ABOVE HTTP PROXY
        });
    }
    listen(port) {
        var _a, _b, _c, _d;
        if (typeof port === 'number') {
            this.bridgeServer.listen(port);
            this.httpServer.listen(0);
            (_a = this.httpsServer) === null || _a === void 0 ? void 0 : _a.listen(0);
        }
        else {
            this.bridgeServer.listen(port.bridgePort);
            this.httpServer.listen((_b = port.httpPort) !== null && _b !== void 0 ? _b : 0);
            (_c = this.httpsServer) === null || _c === void 0 ? void 0 : _c.listen((_d = port.httpsPort) !== null && _d !== void 0 ? _d : 0);
        }
    }
    close() {
        var _a;
        this.bridgeServer.close();
        this.httpServer.close();
        (_a = this.httpsServer) === null || _a === void 0 ? void 0 : _a.close();
    }
}
exports.default = BridgeServer;
