"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorType = void 0;
var ErrorType;
(function (ErrorType) {
    // Hostname could not be resolved on destination IP address
    ErrorType["invalidHostname"] = "ENOTFOUND";
    // Connection abruptly closed.
    ErrorType["connectionReset"] = "ECONNRESET";
    ErrorType["inconsistency"] = "Inconsistency Error";
    ErrorType["unknown"] = "Unknown Error";
})(ErrorType = exports.ErrorType || (exports.ErrorType = {}));
class ProxyError extends Error {
    constructor(message, type, original) {
        super(message);
        this.type = type;
        this.original = original;
    }
    get code() {
        switch (this.type) {
            case ErrorType.unknown: return 0;
            case ErrorType.inconsistency: return 1;
            case ErrorType.connectionReset: return 2;
            case ErrorType.invalidHostname: return 3;
        }
    }
    static from(error) {
        switch (error.code) {
            case 'ENOTFOUND': {
                const hostname = error.hostname;
                const message = `Failed to resolve ${hostname} ip address`;
                return new ProxyError(message, ErrorType.invalidHostname, error);
            }
            case 'ECONNRESET': {
                const message = `Connection was abruptly closed`;
                return new ProxyError(message, ErrorType.connectionReset, error);
            }
            default: return new ProxyError(`Unknown error`, ErrorType.unknown, error);
        }
    }
}
exports.default = ProxyError;
