"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class RequestModel {
    constructor(incomingMessage = undefined) {
        var _a, _b;
        this.httpVersion = (_a = incomingMessage === null || incomingMessage === void 0 ? void 0 : incomingMessage.httpVersion) !== null && _a !== void 0 ? _a : "";
        this.headers = this.formattedHeaders((_b = incomingMessage === null || incomingMessage === void 0 ? void 0 : incomingMessage.headers) !== null && _b !== void 0 ? _b : {});
        this.body = Buffer.from('');
        this.remoteAddress = '';
        this.size = incomingMessage ? this.calculateSize(incomingMessage) : 0;
    }
    get stringBody() { return this.body.toString('utf-8'); }
    set stringBody(newBody) { this.body = new Buffer(newBody, 'utf-8'); }
    formattedHeaders(headers) {
        const headersDictionary = {};
        for (const key of Object.keys(headers)) {
            const value = headers[key];
            if (value === undefined) {
                continue;
            }
            headersDictionary[key] = value;
        }
        return headersDictionary;
    }
    /**
     * Socket connections may be reused. It means that `connection.bytesRead` may have the size
     * from the last executed request. This workarounds calculate the difference from this size
     * to get the size of the specific request/response
     */
    calculateSize(incomingMessage) {
        const requestConnection = incomingMessage.connection;
        const bytesReadPreviously = requestConnection._requestStats ? requestConnection._requestStats.bytesRead : 0;
        const bytesReadDelta = requestConnection.bytesRead - bytesReadPreviously;
        requestConnection._requestStats = {
            bytesRead: requestConnection.bytesRead,
            bytesWritten: requestConnection.bytesWritten
        };
        return bytesReadDelta;
    }
}
exports.default = RequestModel;
