"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Proxy = void 0;
const events_1 = require("events");
const errors_1 = require("./errors");
const bridgeServer_1 = __importDefault(require("./bridgeServer"));
class Proxy extends events_1.EventEmitter {
    constructor(options) {
        super();
        this.interceptors = new Map();
        const interceptHandler = (phase, request, response) => this.onIntercept(phase, request, response);
        this.bridgeServer = new bridgeServer_1.default(interceptHandler, options.certAuthority);
        this.setupErrorHandling();
    }
    setupErrorHandling() {
        this.bridgeServer.on('error', (error) => {
            if (error instanceof errors_1.ProxyError) {
                this.emit('error', error);
                return;
            }
            const proxyError = errors_1.ProxyError.from(error);
            this.emit('error', proxyError);
        });
    }
    onIntercept(phase, request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            const handler = this.interceptors.get(phase);
            if (!handler) {
                const content = phase === 'request' ? request : response;
                return Promise.resolve(content);
            }
            const modifiedContent = yield handler(request, response);
            return Promise.resolve(modifiedContent);
        });
    }
    intercept(options, handler) {
        this.interceptors.set(options.phase, handler);
    }
    listen(port) {
        this.bridgeServer.listen(port);
    }
    close() {
        this.bridgeServer.close();
    }
}
exports.Proxy = Proxy;
