/// <reference types="node" />
import * as http from 'http';
import { EventEmitter } from 'events';
import { IRequest, IResponse, Request, Response } from './models';
declare type AnyContent = IRequest | IResponse;
declare type InterceptHandler = (phase: string, request: IRequest, response: IResponse) => Promise<AnyContent>;
export default class Server extends EventEmitter {
    private interceptHandler;
    constructor(interceptHandler: InterceptHandler);
    protected sendRequest(req: http.IncomingMessage, res: http.ServerResponse, protocol: string): Promise<void>;
    /**
     * Handles the response from a forwarded request.
     *
     * Receives the response and send it to the client before letting the system receive it.
     * Once the client returns - a modified or not - response, send it to the system.
     *
     * @param request The request made
     * @param response The response that can be send to the client
     * @param serverResponse The response received from the Router
     * @param proxyResponse The proxy response, that will be sent to the system
     */
    protected receiveResponse(request: Request, response: Response, serverResponse: http.IncomingMessage, proxyResponse: http.ServerResponse): Promise<void>;
    private collectMessageBody;
}
export {};
