"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const zlib = __importStar(require("zlib"));
const events_1 = require("events");
const models_1 = require("./models");
const proxy_error_1 = __importStar(require("./errors/proxy-error"));
const router_1 = __importDefault(require("./router"));
class Server extends events_1.EventEmitter {
    constructor(interceptHandler) {
        super();
        this.interceptHandler = interceptHandler;
    }
    sendRequest(req, res, protocol) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = new models_1.Request(req, protocol);
            // At this request phase, `res` is not populated yet
            const response = new models_1.Response();
            try {
                request.body = yield this.collectMessageBody(req);
            }
            catch (error) {
                console.log(error);
                req.destroy(new proxy_error_1.default('Error while fetching request body', proxy_error_1.ErrorType.unknown, error));
                return;
            }
            let modifiedRequest;
            try {
                // Send the intercepted request to the client before forwarding to its destination
                modifiedRequest = (yield this.interceptHandler('request', request, response));
            }
            catch (error) {
                console.log(error);
                req.destroy();
                return;
            }
            const responseHandler = (serverResponse) => __awaiter(this, void 0, void 0, function* () { return this.receiveResponse(request, response, serverResponse, res); });
            const forwardedRequest = router_1.default.forward(modifiedRequest, responseHandler);
            forwardedRequest.on('error', error => this.emit('error', error));
            forwardedRequest.write(modifiedRequest.body);
            forwardedRequest.end();
        });
    }
    /**
     * Handles the response from a forwarded request.
     *
     * Receives the response and send it to the client before letting the system receive it.
     * Once the client returns - a modified or not - response, send it to the system.
     *
     * @param request The request made
     * @param response The response that can be send to the client
     * @param serverResponse The response received from the Router
     * @param proxyResponse The proxy response, that will be sent to the system
     */
    receiveResponse(request, response, serverResponse, proxyResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            response.populate(serverResponse);
            const contentEncoding = serverResponse.headers["content-encoding"];
            let responseContent = serverResponse;
            if (contentEncoding && contentEncoding.toLowerCase().includes("gzip")) {
                const gunzip = zlib.createGunzip();
                serverResponse.pipe(gunzip);
                responseContent = gunzip;
                delete response.headers["content-encoding"];
            }
            else if (contentEncoding && contentEncoding.toLowerCase().includes("deflate")) {
                const deflate = zlib.createInflate();
                serverResponse.pipe(deflate);
                responseContent = deflate;
                delete response.headers["content-encoding"];
            }
            else if (contentEncoding && contentEncoding.toLowerCase().includes("br")) {
                const brotli = zlib.createBrotliDecompress();
                serverResponse.pipe(brotli);
                responseContent = brotli;
                delete response.headers["content-encoding"];
            }
            try {
                response.body = yield this.collectMessageBody(responseContent);
                if (contentEncoding) {
                    response.headers['content-length'] = response.body.byteLength.toString();
                }
            }
            catch (error) {
                console.log(error);
                proxyResponse.destroy(new proxy_error_1.default('Error while fetching response body', proxy_error_1.ErrorType.inconsistency, error));
                return;
            }
            let modifiedResponse;
            try {
                modifiedResponse = (yield this.interceptHandler('response', request, response));
            }
            catch (error) {
                console.log(error);
                proxyResponse.destroy();
                return;
            }
            proxyResponse.writeHead(modifiedResponse.statusCode, modifiedResponse.headers);
            proxyResponse.write(modifiedResponse.body);
            proxyResponse.end();
        });
    }
    collectMessageBody(incomingMessage) {
        return new Promise((resolve, reject) => {
            let bodyBuffers = [];
            incomingMessage.on('data', chunk => bodyBuffers.push(chunk));
            incomingMessage.on('end', () => resolve(Buffer.concat(bodyBuffers)));
            incomingMessage.on('error', error => reject(error));
        });
    }
}
exports.default = Server;
